function [ Dish, DishBW, PetriX, PetriY, R_ROI, TotArea ] = DishPreprocessor( img )
% DishPreprocessor Image preprocessor for Petri Dish processing 
%
% DishPreprocessor processes gray-scale image of round colonies on a Petri
% Dish. Dark background with light, roughly round-shaped colonies is
% assumed.
%
% The function performs the following steps:
% - a threshold to eliminate background around the dish is determined
% - exact position of the dish in the image is determined
% - the inner background is eliminated
% - the radius of the dish in the image is determined
% 
% Outputs: 
% DishBW - B/W image of the colonies
% PetriX, PetriY - coordinates of the center of the dish
% R_ROI - radius of the ROI
%
% Note: image size 560x752, 768x1024 or 1536x2048 is assumed
%
% Author Jan Schier <schier@utia.cas.cz>
% Date 2009/08/04
% 
% Copyright
% (c) 2009 Dept. of Signal Processing
% Institute of Information Theory and Automation
% Academy of Sciences of the Czech Republic, Prague
%
% http://sp.utia.cz
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% You are free to distribute this software under Version 2.1
% of the GNU General Public License.


DimTable = [560 752; ...
            768 1024;
            1536 2048];

Params.width = [28 39 80];

Params.MaxDiam = [9999 9999 1430];

Dish       = []; % Create matrix
DishEroded = []; % Create matrix
BlobCenter = [];
area       = [];

% Size of the image
[sy,sx] = size(img);
% Select appropriate mask parameter set
if sy==DimTable(1,1) && sx==DimTable(1,2)
    DimIdx=1;
elseif sy==DimTable(2,1) && sx==DimTable(2,2)
    DimIdx = 2;
elseif sy==DimTable(3,1) && sx==DimTable(3,2)
    DimIdx = 3;
else
    area=-1; % 'Wrong image dimensions';
    return
end

width = Params.width(DimIdx);
MaxDiam = Params.MaxDiam(DimIdx);
ImgParams.width = width;
% background treshold from samples in the image corners
threshold = min([max(max(double(img(1:50,1:50)))) ...
    max(max(double(img(1:50,end-49:end)))) ...
    max(max(double(img(end-49:end,1:50)))) ...
    max(max(double(img(end-49:end,end-49:end))))]) ...
    +20;

img_thresh = img;
img_thresh(img<threshold) = 0;

% Determine dish boundaries from horizontal and vertical projection
% To avoid background noise, threshold NoiseTH is set to 40 pixels
% (non-zero in one ray of projection)
% empirical value, works fairly good
sy2 = round(sy/2);
ImgBin = img_thresh>0;
ProjX = sum(ImgBin,1);
ProjY = sum(ImgBin,2);
NoiseTH = 40;
PetriXL = find(ProjX>NoiseTH,1,'first'); % X Left
PetriXR = find(ProjX>NoiseTH,1,'last');  % X Right
PetriYD = find(ProjY>NoiseTH,1,'first'); % Y Up
PetriYU = find(ProjY>NoiseTH,1,'last');  % Y Down

if isempty(PetriXL) || isempty(PetriXR) || isempty(PetriYD) || isempty(PetriYU)
    area = -2; % 'Empty image?'
    return
end
    
XWidth = PetriXR-PetriXL+1;
YWidth = PetriYU-PetriYD+1;

if XWidth>MaxDiam || YWidth>XWidth
    area = -3;
    return
end

% Petri dish radius in horizontal direction
Radius = round(XWidth/2); 

% Threshold to cut-off the inner background in the dish
% Computed along the line cutting the image horizontally in two halves,
% across the rim of the dish (takes 'width' pixels of the rim 
% and its neighbourhood)
Idx = [PetriXL:PetriXL+width PetriXR-width+1:PetriXR];
ImgMean = round(mean(img_thresh(sy2,Idx)));
threshold = ImgMean;

% cut-off the inner background
img_thresh = img_thresh>threshold;

% Dish center and radius calculation
if PetriXL==1 || PetriXR==sx || PetriYD==1 || PetriYU==sy 
    area = -4; % 'Background corrupted or dish out of center'
    return
end

% 'Light' version for demo, in full version, dish with rim out of image is
% also resolved
PetriX = PetriXL+Radius-1;

YWidth = PetriYU-PetriYD+1;
PetriY = PetriYD+round(YWidth/2)-1;

% [PetriX, PetriY] = DishPosition(ImgBin, Radius, PetriXL, PetriXR, PetriYD, PetriYU);


%% Dish diameter

% Outer radius of the dish
R2Measured = Radius;
% Radius of the inner part, w/o dish rim
R1Measured = R2Measured-width;

%% Mask ROI and count the colonies
R_ROI = R1Measured;
DishDiam = 2*R_ROI+1;
TotArea = 0;

% DishBW and Dish are square matrices assigned 
% the circular inner area (ROI) of the Petri dish
% DishBW contains binary image, Dish gray scale image of the colonies,
% w/o background
DishBW  =zeros(DishDiam,DishDiam); % BW image
Dish    =zeros(DishDiam,DishDiam); % original image, limited to ROI

% Check eventual image cut-out (dimensions)
if PetriY-R_ROI<1 || PetriY+R_ROI>sy
    area=-5; % 'Dish partly out of image'
    return
end

% Assign the center line of the dish
DishBW(R_ROI+1,:) = img_thresh(PetriY,PetriX+(-R_ROI:R_ROI));
Dish(R_ROI+1,:)   = img(PetriY,PetriX+(-R_ROI:R_ROI));

% Total area of the dish is computed by counting the pixels in each line
TotArea = 2*R_ROI+1;

DishMask = CircleSeg45(R1Measured, 0, 0);
DishMask = [DishMask zeros(1,R1Measured-length(DishMask))];
tmp = CircleSeg45(R1Measured, 0, 1);
DishMask(end-length(tmp)+1:end) = tmp(end:-1:1);

for i=1:R_ROI
    DishBW(R_ROI+1-i,R_ROI+1+(-DishMask(i):DishMask(i))) = ...
        img_thresh(PetriY-i,PetriX+(-(DishMask(i)):DishMask(i)));
    DishBW(R_ROI+1+i,R_ROI+1+(-DishMask(i):DishMask(i))) = ...
        img_thresh(PetriY+i,PetriX+(-(DishMask(i)):DishMask(i)));
    Dish(R_ROI+1-i,R_ROI+1+(-DishMask(i):DishMask(i))) = ...
        img(PetriY-i,PetriX+(-(DishMask(i)):DishMask(i)));
    Dish(R_ROI+1+i,R_ROI+1+(-DishMask(i):DishMask(i))) = ...
        img(PetriY+i,PetriX+(-(DishMask(i)):DishMask(i)));
    TotArea = TotArea + 2*(2*DishMask(i)+1);
end


end

